@@ -5,7 +5,8 @@ import numba
 from numba.typed import List
 
 from exetera.core import validation as val
-from exetera.core import utils
+from exetera.core.abstract_types import Field
+from exetera.core import fields, utils
 
 DEFAULT_CHUNKSIZE = 1 << 20
 INVALID_INDEX = 1 << 62
@@ -20,14 +21,62 @@ def chunks(length, chunksize=1 << 20):
         cur = next_
 
 
+def safe_map(field, map_field, map_filter, empty_value=None):
+    if isinstance(field, Field):
+        if field.indexed:
+            return safe_map_indexed_values(
+                field.indices[:], field.values[:], map_field, map_filter, empty_value)
+        else:
+            return safe_map_values(field.data[:], map_field, map_filter, empty_value)
+    elif isinstance(field, np.ndarray):
+        return safe_map_values(field, map_field, map_filter, empty_value)
+
+
 @njit
-def safe_map(data_field, map_field, map_filter, empty_value):
+def safe_map_indexed_values(data_indices, data_values, map_field, map_filter, empty_value=None):
+    empty_value_len = 0 if empty_value is None else len(empty_value)
+    value_length = 0
+    for i in range(len(map_field)):
+        if map_filter[i]:
+            value_length += data_indices[map_field[i]+1] - data_indices[map_field[i]]
+        else:
+            value_length += empty_value_len
+
+    i_result = np.zeros(len(map_field) + 1, dtype=data_indices.dtype)
+    v_result = np.zeros(value_length, dtype=data_values.dtype)
+
+    offset = 0
+    i_result[0] = 0
+    for i in range(len(map_field)):
+        if map_filter[i]:
+            sst = data_indices[map_field[i]]
+            sse = data_indices[map_field[i]+1]
+            dst = offset
+            delta = sse - sst
+            dse = offset + delta
+            i_result[i+1] = dse
+            v_result[dst:dse] = data_values[sst:sse]
+            offset += delta
+        else:
+            dst = offset
+            dse = offset + empty_value_len
+            i_result[i+1] = dse
+            if empty_value is not None:
+                v_result[dst:dse] = empty_value
+            offset += dse - dst
+
+    return i_result, v_result
+
+
+@njit
+def safe_map_values(data_field, map_field, map_filter, empty_value=None):
     result = np.zeros_like(map_field, dtype=data_field.dtype)
+    empty_val = result[0] if empty_value is None else empty_value
     for i in range(len(map_field)):
         if map_filter[i]:
             result[i] = data_field[map_field[i]]
         else:
-            result[i] = empty_value
+            result[i] = empty_val
     return result
 
 
@@ -103,7 +152,6 @@ def data_iterator(data_field, chunksize=1 << 20):
         for v in range(c[0], c[1]):
             yield data[v]
 
-
 @njit
 def apply_filter_to_index_values(index_filter, indices, values):
     # pass 1 - determine the destination lengths
@@ -158,6 +206,66 @@ def apply_indices_to_index_values(indices_to_apply, indices, values):
     return dest_indices, dest_values
 
 
+def get_spans_for_field(ndarray):
+    results = np.zeros(len(ndarray) + 1, dtype=bool)
+    if np.issubdtype(ndarray.dtype, np.number):
+        fn = np.not_equal
+    else:
+        fn = np.char.not_equal
+    results[1:-1] = fn(ndarray[:-1], ndarray[1:])
+
+    results[0] = True
+    results[-1] = True
+    return np.nonzero(results)[0]
+
+
+@njit
+def _get_spans_for_2_fields_by_spans(span0, span1):
+    spans = []
+    j=0
+    for i in range(len(span0)):
+        if j<len(span1):
+            while span1[j] < span0[i]:
+                spans.append(span1[j])
+                j += 1
+            if span1[j] == span0[i]:
+                j += 1
+        spans.append(span0[i])
+    if j<len(span1): #if two ndarray are not equally sized
+        spans.extend(span1[j:])
+    return spans
+
+
+@njit
+def _get_spans_for_2_fields(ndarray0, ndarray1):
+    count = 0
+    spans = np.zeros(len(ndarray0)+1, dtype=np.uint32)
+    spans[0] = 0
+    for i in np.arange(1, len(ndarray0)):
+        if ndarray0[i] != ndarray0[i-1] or ndarray1[i] != ndarray1[i-1]:
+            count += 1
+            spans[count] = i
+    spans[count+1] = len(ndarray0)
+    return spans[:count+2]
+
+
+@njit
+def _get_spans_for_index_string_field(indices,values):
+    result = []
+    result.append(0)
+    for i in range(1, len(indices) - 1):
+        last = indices[i - 1]
+        current = indices[i]
+        next = indices[i + 1]
+        if next - current != current - last:  # compare size first
+            result.append(i)
+            continue
+        if not np.array_equal(values[last:current], values[current:next]):
+            result.append(i)
+    result.append(len(indices) - 1)  # total number of elements
+    return result
+
+
 @njit
 def apply_spans_index_of_min(spans, src_array, dest_array):
     for i in range(len(spans)-1):
@@ -172,6 +280,84 @@ def apply_spans_index_of_min(spans, src_array, dest_array):
     return dest_array
 
 
+@njit
+def apply_spans_index_of_min_indexed(spans, src_indices, src_values, dest_array):
+    for i in range(len(spans)-1):
+        cur = spans[i]
+        next = spans[i+1]
+
+        if next - cur == 1:
+            dest_array[i] = cur
+        else:
+            minind = cur
+            minstart = src_indices[cur]
+            minend = src_indices[cur+1]
+            minlen = minend - minstart
+            for j in range(cur+1, next):
+                curstart = src_indices[j]
+                curend = src_indices[j+1]
+                curlen = curend - curstart
+                shortlen = min(curlen, minlen)
+                found = False
+                for k in range(shortlen):
+                    if src_values[curstart+k] < src_values[minstart+k]:
+                        minind = j
+                        minstart = curstart
+                        minend = curend
+                        found = True
+                        break
+                    elif src_values[curstart+k] > src_values[minstart+k]:
+                        found = True
+                        break
+                if not found and curlen < minlen:
+                    minind = j
+                    minstart = curstart
+                    minend = curend
+
+            dest_array[i] = minind
+
+    return dest_array
+
+
+@njit
+def apply_spans_index_of_max_indexed(spans, src_indices, src_values, dest_array):
+    for i in range(len(spans)-1):
+        cur = spans[i]
+        next = spans[i+1]
+
+        if next - cur == 1:
+            dest_array[i] = cur
+        else:
+            minind = cur
+            minstart = src_indices[cur]
+            minend = src_indices[cur+1]
+            minlen = minend - minstart
+            for j in range(cur+1, next):
+                curstart = src_indices[j]
+                curend = src_indices[j+1]
+                curlen = curend - curstart
+                shortlen = min(curlen, minlen)
+                found = False
+                for k in range(shortlen):
+                    if src_values[curstart+k] > src_values[minstart+k]:
+                        minind = j
+                        minstart = curstart
+                        minlen = curend - curstart
+                        found = True
+                        break
+                    elif src_values[curstart+k] < src_values[minstart+k]:
+                        found = True
+                        break
+                if not found and curlen > minlen:
+                    minind = j
+                    minstart = curstart
+                    minlen = curend - curstart
+
+            dest_array[i] = minind
+
+    return dest_array
+
+
 @njit
 def apply_spans_index_of_max(spans, src_array, dest_array):
     for i in range(len(spans)-1):
@@ -617,6 +803,7 @@ def ordered_inner_map_result_size(left, right):
     return result_size
 
 
+@njit
 def ordered_outer_map_result_size_both_unique(left, right):
     i = 0
     j = 0
@@ -1036,7 +1223,7 @@ def streaming_sort_merge(src_index_f, src_value_f, tgt_index_f, tgt_value_f,
         tgt_value_f.data.write(dest_values[:index_delta])
         target_index += index_delta
 
-        chunk_filter = np.ones(segment_count, dtype=np.bool)
+        chunk_filter = np.ones(segment_count, dtype=bool)
         for i in range(segment_count):
             if in_chunk_indices[i] == in_chunk_lengths[i]:
                 chunk_indices[i] += 1
@@ -1096,3 +1283,14 @@ def streaming_sort_partial(in_chunk_indices, in_chunk_lengths,
         in_chunk_indices[min_value_index] += 1
 
     return dest_index
+
+
+def is_ordered(field):
+    if len(field) == 1:
+        return True
+
+    if np.issubdtype(field.dtype, np.number):
+        fn = np.greater
+    else:
+        fn = np.char.greater
+    return not np.any(fn(field[:-1], field[1:]))
